import logging as lg
import sys
import os

class SgLogSetup:
    """
    Wraps the default logger functionality in a friendly interface, which reflects our problem-domain, namely recording
    activity of short-lived scripts for both prod and dev-env debugging.
    """

    def __init__(self):
        """
        Creates the SgLogSetup with the default attributes.

        NOTE: This sets the `logger.root` log-level to `logger.NOTSET == 0`, which enables arbitrary level setting for
              loggers created off of it.
        """
        lg.root.setLevel(lg.NOTSET)

        self.__formatter = lg.Formatter('[%(asctime)s] [%(name)s] [%(levelname)s] %(message)s')
        self.__logfile = None

    def change_log_format(self, log_format: str) -> None:
        """
        Changes the logfile format from the default one to a custom one.
        This will affect loggers created after the change has taken place.
        The default format is:
        `[%(levelname)s] [%(asctime)s] [%(name)s] %(message)s`
        """
        self.__formatter = lg.Formatter(log_format)

    def get_logger(self,
                   logger_name: str = "scraper",
                   logfile_path: str = "/tmp/scraper.log",
                   stdout_log_level: int = lg.DEBUG,
                   logfile_log_level: int = lg.INFO) -> lg.Logger:
        """
        Uses the underlying `logging.getLogger` method to fetch a logger by its name, set two handlers on it
        (stdout and file), and return said logger to the caller.

        :param logger_name: The name of the logger. Will be visible in the logfile.
        :param logfile_path: The filesystem path of the logfile. Defaults to `/tmp/scraper.log`
        :param stdout_log_level: Minimum log level for the standard-out logging. Mirrors log levels set in the `logging` library.
                                 Defaults to `logger.DEBUG` for debugging during development.
                                 Will be overridden by the `SG_LOG_LVL_STDOUT` env var.
        :param logfile_log_level: Minimum log level for the logfile. Mirrors log levels set in the `logging` library.
                                  Defaults to `logger.INFO` as a reasonable compromise.
                                  Will be overridden by the `SG_LOG_LVL_FILE` env var.

        :return: The logger, as specified above.
        """
        env_loglevel_stdout = os.environ.get('SG_LOG_LVL_STDOUT')
        env_loglevel_file = os.environ.get('SG_LOG_LVL_FILE')

        logger = lg.getLogger(logger_name)

        stdout = lg.StreamHandler(sys.stdout)
        stdout.setLevel(int(env_loglevel_stdout) if env_loglevel_stdout else stdout_log_level)
        stdout.setFormatter(self.__formatter)
        logger.addHandler(stdout)

        logfile = lg.FileHandler(logfile_path)
        logfile.setLevel(int(env_loglevel_file) if env_loglevel_file else logfile_log_level)
        logfile.setFormatter(self.__formatter)
        logger.addHandler(logfile)
        self.__logfile = logfile # so we can close it after

        return logger


    def close(self):
        """
        Closes and cleans up the logfile handler.
        """
        self.__logfile.close()